@@ -53,6 +53,21 @@ Module for importing data files.
 from __future__ import (absolute_import, division, print_function,
                         unicode_literals)
 
+import numpy as np
+import six
+import os
+import h5py
+import logging
+import re
+import math
+import struct
+from contextlib import contextmanager
+import dxchange.writer as writer
+from dxchange.dtype import empty_shared_array
+import warnings
+import functools
+import tifffile
+import scipy.misc as sm
 
 __author__ = "Doga Gursoy, Francesco De Carlo"
 __copyright__ = "Copyright (c) 2015-2016, UChicago Argonne, LLC."
@@ -68,20 +83,9 @@ __all__ = ['read_edf',
            'read_tiff_stack',
            'read_xrm',
            'read_xrm_stack',
-           'read_hdf5_stack']
-
-
-import numpy as np
-import six
-import os
-import h5py
-import logging
-import re
-import math
-import struct
-from contextlib import contextmanager
-import dxchange.writer as writer
-from dxchange.dtype import empty_shared_array
+           'read_txrm',
+           'read_hdf5_stack',
+           'read_file_list']
 
 logger = logging.getLogger(__name__)
 
@@ -100,12 +104,12 @@ EdfFile = _check_import('EdfFile')
 astropy = _check_import('astropy')
 olefile = _check_import('olefile')
 
-
 # FIXME: raise exception would make more sense, also not sure an extension check
 # is very useful, unless we are automatically mapping an extension to a
 # function.
 def _check_read(fname):
-    known_extensions = ['.edf', '.tiff', '.tif', '.h5', '.hdf', '.npy', '.xrm']
+    known_extensions = ['.edf', '.tiff', '.tif', '.h5', '.hdf', '.npy', '.nc', '.xrm',
+                        '.txrm', '.txm']
     if not isinstance(fname, six.string_types):
         logger.error('File name must be a string')
     else:
@@ -134,7 +138,6 @@ def read_tiff(fname, slc=None):
     """
     fname = _check_read(fname)
     try:
-        import tifffile
         arr = tifffile.imread(fname, memmap=True)
     except IOError:
         logger.error('No such file or directory: %s', fname)
@@ -144,7 +147,7 @@ def read_tiff(fname, slc=None):
     return arr
 
 
-def read_tiff_stack(fname, ind, digit, slc=None):
+def read_tiff_stack(fname, ind, digit=None, slc=None):
     """
     Read data from stack of tiff files in a folder.
 
@@ -155,7 +158,7 @@ def read_tiff_stack(fname, ind, digit, slc=None):
     ind : list of int
         Indices of the files to read.
     digit : int
-        Number of digits used in indexing stacked files.
+        (Deprecated) Number of digits used in indexing stacked files.
     slc : sequence of tuples, optional
         Range of values for slicing data in each axis.
         ((start_1, end_1, step_1), ... , (start_N, end_N, step_N))
@@ -175,7 +178,8 @@ def read_tiff_stack(fname, ind, digit, slc=None):
     _log_imported_data(fname, arr)
     return arr
 
-def read_xrm(fname, slc=None):
+
+def read_xrm(fname, slice_range=None):
     """
     Read data from xrm file.
 
@@ -183,7 +187,7 @@ def read_xrm(fname, slc=None):
     ----------
     fname : str
         String defining the path of file or file name.
-    slc : sequence of tuples, optional
+    slice_range : sequence of tuples, optional
         Range of values for slicing data in each axis.
         ((start_1, end_1, step_1), ... , (start_N, end_N, step_N))
         defines slicing parameters for each axis of the data matrix.
@@ -201,27 +205,42 @@ def read_xrm(fname, slc=None):
         print('No such file or directory: %s', fname)
         return False
 
-    n_cols = _read_label(ole, 'ImageInfo/ImageWidth', '<I')
-    n_rows = _read_label(ole, 'ImageInfo/ImageHeight', '<I')
-    d_type = _read_label(ole, 'ImageInfo/DataType', '<1I')
+    metadata = read_ole_metadata(ole)
 
-    # 10 float; 5 uint16 (unsigned 16-bit (2-byte) integers)
-    if d_type == 10:
-        struct_fmt = "<{}f".format(n_cols*n_rows)
-    elif d_type == 5:                   
-        struct_fmt = "<{}h".format(n_cols*n_rows)
-    
-    img = _read_data(ole, "ImageData1/Image1", struct_fmt)
+    arr = np.empty(
+        _shape_after_slice(
+            (metadata["image_width"], metadata["image_height"]),
+            slice_range,
+        ),
+        dtype=np.float32
+    )
 
-    arr = np.zeros((n_cols, n_rows))    
-    arr[:,:] = np.reshape(img, (n_cols, n_rows), order='F')
-    arr = np.swapaxes(arr,0,1)
+    if slice_range is None:
+        slice_range = (slice(None), slice(None))
+    else:
+        slice_range = _make_slice_object_a_tuple(slice_range)
+
+    stream = ole.openstream("ImageData1/Image1")
+    data = stream.read()
+
+    data_type = _get_ole_data_type(metadata)
+    data_type = data_type.newbyteorder('<')
+
+    arr = np.reshape(
+        np.fromstring(data, data_type),
+        (
+            metadata["image_width"],
+            metadata["image_height"]
+        )
+    )[slice_range]
 
-    arr = _slice_array(arr, slc)  
     _log_imported_data(fname, arr)
-    return arr
 
-def read_xrm_stack(fname, ind, digit, slc=None):
+    ole.close()
+    return arr, metadata
+
+#  Should slc just take over what ind is doing here?
+def read_xrm_stack(fname, ind, slc=None):
     """
     Read data from stack of xrm files in a folder.
 
@@ -231,8 +250,6 @@ def read_xrm_stack(fname, ind, digit, slc=None):
         One of the file names in the tiff stack.
     ind : list of int
         Indices of the files to read.
-    digit : int
-        Number of digits used in indexing stacked files.
     slc : sequence of tuples, optional
         Range of values for slicing data in each axis.
         ((start_1, end_1, step_1), ... , (start_N, end_N, step_N))
@@ -244,39 +261,187 @@ def read_xrm_stack(fname, ind, digit, slc=None):
         Output 3D image.
     """
     fname = _check_read(fname)
-    list_fname = _list_file_stack(fname, ind, digit)
+    list_fname = _list_file_stack(fname, ind)
+
+    number_of_images = len(ind)
+    arr, metadata = _init_ole_arr_from_stack(
+        list_fname[0], number_of_images, slc)
+    del metadata["thetas"][0]
+    del metadata["x_positions"][0]
+    del metadata["y_positions"][0]
 
-    arr = _init_ole_arr_from_stack(list_fname[0], len(ind), slc)
     for m, fname in enumerate(list_fname):
-        arr[m] = read_xrm(fname, slc)
+        arr[m], angle_metadata = read_xrm(fname, slc)
+        metadata["thetas"].append(angle_metadata["thetas"][0])
+        metadata["x_positions"].append(angle_metadata["x_positions"][0])
+        metadata["y_positions"].append(angle_metadata["y_positions"][0])
+
     _log_imported_data(fname, arr)
-    return arr
+    return arr, metadata
+
+
+def read_txrm(file_name, slice_range=None):
+    """
+    Read data from a .txrm file, a compilation of .xrm files.
+
+    Parameters
+    ----------
+    file_name : str
+        String defining the path of file or file name.
+    slice_range : sequence of tuples, optional
+        Range of values for slicing data in each axis.
+        ((start_1, end_1, step_1), ... , (start_N, end_N, step_N))
+        defines slicing parameters for each axis of the data matrix.
+
+    Returns
+    -------
+    ndarray
+        Array of 2D images.
+
+    dictionary
+        Dictionary of metadata.
+    """
+    file_name = _check_read(file_name)
+    try:
+        import olefile
+        ole = olefile.OleFileIO(file_name)
+    except IOError:
+        print('No such file or directory: %s', file_name)
+        return False
+
+    metadata = read_ole_metadata(ole)
+
+    array_of_images = np.empty(
+        _shape_after_slice(
+            (
+                metadata["number_of_images"],
+                metadata["image_height"],
+                metadata["image_width"],
+            ),
+            slice_range
+        ),
+        dtype=np.float32
+    )
+
+    if slice_range is None:
+        slice_range = (slice(None), slice(None), slice(None))
+    else:
+        slice_range = _make_slice_object_a_tuple(slice_range)
+
+    for i in range(*slice_range[0].indices(metadata["number_of_images"])):
+        img_string = "ImageData{}/Image{}".format(
+            int(np.ceil((i + 1) / 100.0)), int(i + 1))
+        stream = ole.openstream(img_string)
+        data = stream.read()
+
+        data_type = _get_ole_data_type(metadata)
+
+        data_type = data_type.newbyteorder('<')
+        array_of_images[i] = np.reshape(
+            np.fromstring(data, data_type),
+            (metadata["image_height"], metadata["image_width"], )
+        )[slice_range[1:]]
+
+    _log_imported_data(file_name, array_of_images)
+
+    ole.close()
+    return array_of_images, metadata
+
+
+def read_txm(file_name, slice_range=None):
+    """
+    Read data from a .txm file, the reconstruction file output
+    by Zeiss software.
+
+    Parameters
+    ----------
+    file_name : str
+        String defining the path of file or file name.
+    slice_range : sequence of tuples, optional
+        Range of values for slicing data in each axis.
+        ((start_1, end_1, step_1), ... , (start_N, end_N, step_N))
+        defines slicing parameters for each axis of the data matrix.
+
+    Returns
+    -------
+    ndarray
+        Array of 2D images.
+
+    dictionary
+        Dictionary of metadata.
+    """
+
+    return read_txrm(file_name, slice_range)
+
+
+def read_ole_metadata(ole):
+    """
+    Read metadata from an xradia OLE file (.xrm, .txrm, .txm).
+
+    Parameters
+    ----------
+    ole : OleFileIO instance
+        An ole file to read from.
+
+    Returns
+    -------
+    tuple
+        A tuple of image metadata.
+    """
+
+    number_of_images = _read_ole_data(ole, "ImageInfo/NoOfImages", "<I")[0]
+
+    metadata = {
+        'facility': _read_ole_data(ole, 'SampleInfo/Facility', '<50s'),
+        'image_width': _read_label(ole, 'ImageInfo/ImageWidth', '<I'),
+        'image_height': _read_label(ole, 'ImageInfo/ImageHeight', '<I'),
+        'data_type': _read_label(ole, 'ImageInfo/DataType', '<1I'),
+        'number_of_images': number_of_images,
+        'thetas': list(_read_ole_data(
+            ole, 'ImageInfo/Angles', "<{0}f".format(number_of_images))),
+        'x_positions': list(_read_ole_data(
+            ole, 'ImageInfo/XPosition', "<{0}f".format(number_of_images))),
+        'y_positions': list(_read_ole_data(
+            ole, 'ImageInfo/YPosition', "<{0}f".format(number_of_images)))
+    }
+    return metadata
 
 
 def _log_imported_data(fname, arr):
     logger.debug('Data shape & type: %s %s', arr.shape, arr.dtype)
-    logger.info('Data succesfully imported: %s', fname)
+    logger.info('Data successfully imported: %s', fname)
 
 
-def _init_arr_from_stack(fname, nfile, slc):
+def _init_arr_from_stack(fname, number_of_files, slc):
     """
     Initialize numpy array from files in a folder.
     """
     _arr = read_tiff(fname, slc)
-    size = (nfile, _arr.shape[0], _arr.shape[1])
+    size = (number_of_files, _arr.shape[0], _arr.shape[1])
     logger.debug('Data initialized with size: %s', size)
-    return np.zeros(size, dtype=_arr.dtype)
+    return np.empty(size, dtype=_arr.dtype)
 
-def _init_ole_arr_from_stack(fname, nfile, slc):
+
+def _init_ole_arr_from_stack(fname, number_of_files, slc):
     """
     Initialize numpy array from files in a folder.
     """
-    _arr = read_xrm(fname, slc)
-    size = (nfile, _arr.shape[0], _arr.shape[1])
+    _arr, metadata = read_xrm(fname, slc)
+    size = (number_of_files, _arr.shape[0], _arr.shape[1])
     logger.debug('Data initialized with size: %s', size)
-    return np.zeros(size, dtype=_arr.dtype)
+    return np.empty(size, dtype=_arr.dtype), metadata
 
 
+def _get_ole_data_type(metadata):
+    # 10 float; 5 uint16 (unsigned 16-bit (2-byte) integers)
+    if metadata["data_type"] == 10:
+        return np.dtype(np.float32)
+    elif metadata["data_type"] == 5:
+        return np.dtype(np.uint16)
+    else:
+        print("Wrong data type")
+        return False
+
 def read_edf(fname, slc=None):
     """
     Read data from edf file.
@@ -310,7 +475,34 @@ def read_edf(fname, slc=None):
     return arr
 
 
-def read_hdf5(fname, dataset, slc=None, dtype=None, shared=True):
+def read_dx_dims(fname, dataset):
+    """
+    Read array size of a specific group of Data Exchange file.
+    Parameters
+    ----------
+    fname : str
+        String defining the path of file or file name.
+    dataset : str
+        Path to the dataset inside hdf5 file where data is located.
+    Returns
+    -------
+    ndarray
+        Data set size.
+    """
+
+    grp = '/'.join(['exchange', dataset])
+
+    with h5py.File(fname, "r") as f:
+        try:
+            data = f[grp]
+        except KeyError:
+            return None
+
+        shape = data.shape
+
+    return shape
+
+def read_hdf5(fname, dataset, slc=None, dtype=None, shared=False):
     """
     Read data from hdf5 file from a specific group.
 
@@ -351,9 +543,9 @@ def read_hdf5(fname, dataset, slc=None, dtype=None, shared=True):
                 arr = empty_shared_array(shape, dtype)
             else:
                 arr = np.empty(shape, dtype)
-            data.read_direct(arr, _fix_slice(slc))
+            data.read_direct(arr, _make_slice_object_a_tuple(slc))
     except KeyError:
-        arr = None
+        return None
     _log_imported_data(fname, arr)
     return arr
 
@@ -443,19 +635,18 @@ def read_spe(fname, slc=None):
     return arr
 
 
-def _fix_slice(slc):
+def _make_slice_object_a_tuple(slc):
     """
     Fix up a slc object to be tuple of slices.
-    slc = None is treated as no slc
+    slc = None returns None
     slc is container and each element is converted into a slice object
-    None is treated as slice(None)
 
     Parameters
     ----------
     slc : None or sequence of tuples
         Range of values for slicing data in each axis.
         ((start_1, end_1, step_1), ... , (start_N, end_N, step_N))
-        defines slicing parameters for each axis of the data matrix.  
+        defines slicing parameters for each axis of the data matrix.
     """
     if slc is None:
         return None  # need arr shape to create slice
@@ -548,14 +739,14 @@ def _slice_array(arr, slc):
     if slc is None:
         logger.debug('No slicing applied to image')
         return arr[:]
-    axis_slice = _fix_slice(slc)
+    axis_slice = _make_slice_object_a_tuple(slc)
     logger.debug('Data sliced according to: %s', axis_slice)
     return arr[axis_slice]
 
 
 def _shape_after_slice(shape, slc):
     """
-    Return the calculated shape of an array after it has been sliced.  
+    Return the calculated shape of an array after it has been sliced.
     Only handles basic slicing (not advanced slicing).
 
     Parameters
@@ -570,7 +761,7 @@ def _shape_after_slice(shape, slc):
     if slc is None:
         return shape
     new_shape = list(shape)
-    slc = _fix_slice(slc)
+    slc = _make_slice_object_a_tuple(slc)
     for m, s in enumerate(slc):
         # indicies will perform wrapping and such for the shape
         start, stop, step = s.indices(shape[m])
@@ -580,7 +771,7 @@ def _shape_after_slice(shape, slc):
     return tuple(new_shape)
 
 
-def _list_file_stack(fname, ind, digit):
+def _list_file_stack(fname, ind, digit=None):
     """
     Return a stack of file names in a folder as a list.
 
@@ -591,16 +782,26 @@ def _list_file_stack(fname, ind, digit):
     ind : list of int
         Indices of the files to read.
     digit : int
-        Number of digits in indexing stacked files.
+        Deprecated input for the number of digits in all indexes
+        of the stacked files.
     """
 
-    body = writer.get_body(fname, digit)
+    if digit is not None:
+        warnings.warn(("The 'digit' argument is deprecated and no longer used."
+                      "  It may be removed completely in a later version."),
+                      FutureWarning)
+
+    body = writer.get_body(fname)
+    body, digits = writer.remove_trailing_digits(body)
+
     ext = writer.get_extension(fname)
     list_fname = []
     for m in ind:
-        list_fname.append(str(body + '{0:0={1}d}'.format(m, digit) + ext))
+        counter_string = str(m).zfill(digits)
+        list_fname.append(body + counter_string + ext)
     return list_fname
 
+
 @contextmanager
 def find_dataset_group(fname):
     """
@@ -611,7 +812,7 @@ def find_dataset_group(fname):
     ----------
     fname : str
         String defining the path of file or file name.
-    
+
     Returns
     -------
     h5py.Group
@@ -625,8 +826,8 @@ def _find_dataset_group(h5object):
     Finds the group name containing the stack of projections datasets within
     an ALS BL8.3.2 hdf5 file  containing a stack of images
     """
-    
-    # Only one root key means only one dataset in BL8.3.2 current format    
+
+    # Only one root key means only one dataset in BL8.3.2 current format
     keys = list(h5object.keys())
     if len(keys) == 1:
         if isinstance(h5object[keys[0]], h5py.Group):
@@ -675,7 +876,7 @@ def _map_loc(ind, loc):
     min_loc = low - buff
     max_loc = upp + buff
     loc = np.intersect1d(loc[loc > min_loc], loc[loc < max_loc])
-    new_upp = len(ind)
+    new_upp = len(ind) - 1
     loc = (new_upp * (loc - low)) // (upp - low)
     if loc[0] < 0:
         loc[0] = 0
@@ -687,28 +888,29 @@ def _read_label(ole, label, struct_fmt):
     """
     Reads the integer value associated with label in an ole file
     """
-    
-    if ole.exists(label):   
+
+    if ole.exists(label):
         stream = ole.openstream(label)
         data = stream.read()
         nev = struct.unpack(struct_fmt, data)
         value = np.int(nev[0])
-    
+
     return value
 
-def _read_data(ole, label, struct_fmt):
+
+def _read_ole_data(ole, label, struct_fmt):
     """
     Reads the array associated with label in an ole file
     """
 
-    if ole.exists(label):   
+    if ole.exists(label):
         stream = ole.openstream(label)
         data = stream.read()
         arr = struct.unpack(struct_fmt, data)
-    
+
     return arr
-    
-    
+
+
 def read_hdf5_stack(h5group, dname, ind, digit=4, slc=None, out_ind=None):
     """
     Read data from stacked datasets in a hdf5 file
@@ -723,7 +925,7 @@ def read_hdf5_stack(h5group, dname, ind, digit=4, slc=None, out_ind=None):
         Indices of the datasets to be read
 
     digit : int
-        Number of digits indexing the stacked datasets
+        (Deprecated) Number of digits indexing the stacked datasets
 
     slc : {sequence, int}
         Range of values for slicing data.
@@ -744,7 +946,7 @@ def read_hdf5_stack(h5group, dname, ind, digit=4, slc=None, out_ind=None):
             fname = (writer.get_body(name).split('/')[-1] + '_' + digit * '0' +
                      writer.get_extension(name))
             list_fname_.extend(_list_file_stack(fname, out_ind, digit))
-        list_fname = list_fname_
+        list_fname = sorted(list_fname_, key=lambda x: str(x).split('_')[-1])
 
     for m, image in enumerate(list_fname):
         _arr = h5group[image]
@@ -756,3 +958,33 @@ def read_hdf5_stack(h5group, dname, ind, digit=4, slc=None, out_ind=None):
         arr[m] = _arr
 
     return arr
+
+def read_file_list(file_list):
+    """
+    Read data from stack of image files in a folder.
+
+    Parameters
+    ----------
+
+    file_list : list of str
+        List of file names to read, in order
+    """
+    
+    f = file_list[0]
+    try:
+        readfunc = tifffile.imread
+        im = readfunc(f)
+    except ValueError:
+        readfunc = functools.partial(sm.imread, flatten=True)
+        im = readfunc(f)
+    
+    if len(im.shape) != 2:
+        raise ValueError('Only 2D images are supported in read_file_list')
+
+    arr = np.zeros((len(file_list), im.shape[0], im.shape[1]), dtype=im.dtype)
+
+    arr[0] = im
+    for i, fn in enumerate(file_list[1:]):
+        arr[i+1] = readfunc(fn)
+    
+    return arr
